--[[ Loading relevant libraries ]]--
paddle=require("Library/paddle")
hc = require("Library/HC-master")
particle=require("Library/Particle")
shine=require("Library/shine-master")

touchDebounce=.2 -- debug touch debounce
deathWait=2 -- how long it waits before restarting 
--[[
Main 
--]]
local particles={} 
function love.load()
	font = love.graphics.newFont("Assets/CODE Bold.otf",32)
	pad = paddle.new(400,300)
	_G.padRot=paddle.rot
	ball=require("Library/ball").spawn() -- spawn ball
	circleRadius=150
	blur=shine.gaussianblur()
	blur.sigma=4
	blur2=shine.gaussianblur()
	blur2.sigma=1
	hits=0
	playing=false
	paused=false
	padCollision=hc.rectangle(pad.x,pad.y,pad.w,pad.h)
	ballCollision=hc.rectangle(ball.x,ball.y,ball.w,ball.h)
    love.window.setMode(800, 600, {resizable=false, vsync=true, minwidth=800, minheight=600,msaa=8})
end

function love.draw()
	love.graphics.setBackgroundColor(100,100,100)
	love.graphics.setColor(200,200,200)
	love.graphics.setFont(font)
	blur:draw(function()
		love.graphics.setLineWidth(15)
		love.graphics.circle("line",400,300,150)
	end)
	love.graphics.setColor(255,255,255)
	blur2:draw(function()
		love.graphics.setLineWidth(.1)
		love.graphics.circle("line",400,300,150)
	end)
	love.graphics.setColor(255,255,255)
	if not paused and playing then
		love.graphics.print(hits,400-8,300-8,0)
	end
	love.graphics.setColor(200,200,200)
	pad:draw()
	love.graphics.setColor(255,255,255)
	ball:draw()
	if not playing then
		love.graphics.setColor(255,255,255)
		love.graphics.print(not paused and "- CLICK TO START -" or "- CLICK TO RESUME -",257,280)
	end
	for _, aParticle in pairs(particles) do
		love.graphics.setColor(255,255,255)
		aParticle:draw()
	end
end

function love.mousepressed(x, y, button, istouch)
	if button == 1 then -- Versions prior to 0.10.0 use the MouseConstant 'l'
		if not paused and playing then
			paused=true
			playing=false
		elseif not playing and paused then
			paused=false
			playing=true
		end
		if not playing and not paused then
			playing=true
		end
	end
end


function inCircle(x0,x1,y0,y1)
	return math.sqrt((x1-x0)*(x1-x0) + (y1-y0)*(y1-y0)) < circleRadius
end

function love.update(dt)
	if playing then
		ball:update(dt)
		pad:update(
		function(dt,self)
			local mouseX,mouseY=love.mouse.getPosition()
			self.rot=math.atan2((400 - self.x), -(300 - self.y))
			self.r = math.atan2(mouseY-300, mouseX-400)
			ball.padRot=self.rot
			self.x = circleRadius*math.cos(self.r) + self.orgX;
			self.y = circleRadius*math.sin(self.r) + self.orgY;
		end,
		dt
		)
		
		padCollision:moveTo(pad.x,pad.y)
		padCollision:center()
		padCollision:setRotation(pad.rot+math.rad(90))
		ballCollision:moveTo(ball.x,ball.y)
		ballCollision:setRotation(ball.rot)
		-- print("PadCol:",padCollision:rotation(),"Pad:",pad.rot)
		for shape, delta in pairs(hc.collisions(padCollision)) do
			if touchDebounce == .2 and ball.alive then
				touchDebounce=0.1
				ball:hit(false,delta,pad.rot)
				hits=hits+1
			end
		end
		print(ball.alive)
		if touchDebounce<.2 then
			touchDebounce=touchDebounce-1*dt
		end
		if touchDebounce<=0 then
			touchDebounce = .2
		end
		if not inCircle(ball.x,400,ball.y,300) and ball.alive then
			ball.alive=false
			for i = 1, 20 do
				local expParticle=particle.new(ball.x,ball.y,3,{r=255,g=255,b=255},2)
				expParticle:applyForce({x=math.random(-10,10),y=math.random(-10,10)})
				table.insert(particles,expParticle)
			end
			hits=0
		end
		if ball.alive == false and #particles==0 then
			print("!!")
			ball:reset()
		end
		ball.Speed=200+hits
		for i, expParticles in pairs(particles) do
			expParticles:update(dt)
			if expParticles.alive==false then
				table.remove(particles,i)
			end
		end
	end
end